// 환경 설정 - require( )을 실행하기 전에 정의한다.
require.config({
	//baseUrl:'app', // 모듈을 로딩할 기본 패스를 지정한다, 정의하지 않으면 main.js가 있는 곳이 기본 경로가 된다.

	// 각 모듈의 기본 패스를 지정한다.
	// 모듈의 이름과 실제 경로를 매핑할 수 있어 별칭(alias) 기능도 할 수 있다.
	paths:{
		'jquery' : '../libs/framework/jquery.min',
		'jquery-ui' : '../libs/framework/jquery-ui.min',
		'angular' : '../libs/framework/angular.min',
		'angular-scrollbarconcat': '../libs/amoeba/openModules/jquery.mCustomScrollbar.concat.min',
		'angular-scrollbers': '../libs/amoeba/openModules/scrollbars',
		'angular-slider': '../libs/amoeba/openModules/jquery.ui-slider-master',
		'angular-ui-router' : '../libs/framework/angular-ui-router.min',
		'angular-couch-potato': '../libs/framework/angular-couch-potato',
		'angular-amoeba': '../libs/amoeba/ourModules/angular-amoeba',
		'ng-file-model' : '../libs/amoeba/openModules/ng-file-model',
		'angular-thumbnail' : '../libs/amoeba/openModules/angular-thumbnail'

	},

	// 기본의 코드 배부분은 아마도 define 이나 require을 사용하지 않은 소스코드가 많다. 그리고 그것들도 나름의 의존성을
	// 가지고 있을 수 있다. 이것들을 RequireJS에서도 온전히 사용하려면 그냥은 안된다. 이것을 지원하기 위해 RequireJS
	// 에서는 shim 속성으로 기존 코드의 의존성을 정의할 수 있다.
	shim:{
		'jquery' :{
			exports : 'jquery'
		},
		'jquery-ui' :{
			deps: ['jquery'],
			exports : 'jquery-ui'
		},
		'angular' : {
			// 이름은 위에 이미 지정한 별칭(키 이름)으로 해도 된다.
			deps: ['jquery', 'jquery-ui'],
			exports : 'angular'
		},
		'angular-scrollbarconcat' :{
			deps: ['angular']
		},
		'angular-scrollbers':{
			deps: ['angular', 'angular-scrollbarconcat']
		},
		'angular-slider':{
			deps: ['angular']
		},
		'angular-ui-router':{
			deps: ['angular']
		},
		'angular-amoeba':{
			deps: ['angular']
		},
		'ng-file-model':{
			deps: ['angular']
		},
		'angular-thumbnail':{
			deps: ['angular']
		}
	}
});

// 모듈 정의가 아닌 단순히 코드를 실행할 때 require 함수를 사용한다.
// require은 define과 비슷하게 첫번째 인자로 의존성, 두 번째 인자로 실행 코드 함수를 줄 수 있다.
// define문과 함께 의존성이 중복될 경우, 두 번째에는 모듈을 다시 로딩하지 않고 로딩된 모듈을 다시 사용한다.
require(['ucApp', 'angular', 'angular-scrollbers', 'angular-scrollbarconcat', 'app-init'], function(ucApp, angular) {
	//console.log("require");
	angular.element(document).ready(function() {
  	    console.log("require ready");

	    angular.bootstrap(document, [ucApp['name'], function() {
	        console.log("bootstrap");

		      // for good measure, put ng-app on the html element
		      // studiously avoiding jQuery because angularjs.org says we shouldn't
		      // use it.  In real life, there are a ton of reasons to use it.
		      // karma likes to have ng-app on the html element when using requirejs.
		     angular.element(document).find('html').addClass('ng-app');

	    }]);
    });
});